@@ -2,10 +2,14 @@
 
 __author__ = """François Durand"""
 __email__ = 'fradurand@gmail.com'
-__version__ = '0.14.0'
+__version__ = '0.20.0'
 
 
 # Utils
+from poisson_approval.utils.computation_engine import computation_engine
+from poisson_approval.utils.ComputationEngine import ComputationEngine
+from poisson_approval.utils.ComputationEngineNumeric import ComputationEngineNumeric
+from poisson_approval.utils.ComputationEngineSymbolic import ComputationEngineSymbolic
 from poisson_approval.utils.DictPrintingInOrder import DictPrintingInOrder
 from poisson_approval.utils.DictPrintingInOrderIgnoringNone import DictPrintingInOrderIgnoringNone
 from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
@@ -54,8 +58,10 @@ from poisson_approval.strategies.StrategyOrdinal import StrategyOrdinal
 # Profiles
 from poisson_approval.profiles.Profile import Profile
 from poisson_approval.profiles.ProfileDiscrete import ProfileDiscrete
+from poisson_approval.profiles.ProfileNoisyDiscrete import ProfileNoisyDiscrete
 from poisson_approval.profiles.ProfileOrdinal import ProfileOrdinal
 from poisson_approval.profiles.ProfileCardinal import ProfileCardinal
+from poisson_approval.profiles.ProfileCardinalContinuous import ProfileCardinalContinuous
 from poisson_approval.profiles.ProfileTwelve import ProfileTwelve
 from poisson_approval.profiles.ProfileHistogram import ProfileHistogram
 
@@ -79,3 +85,5 @@ from poisson_approval.generators.GeneratorTauVectorUniform import GeneratorTauVe
 # Meta-analysis
 from poisson_approval.meta_analysis.NiceStatsProfileOrdinal import NiceStatsProfileOrdinal
 from poisson_approval.meta_analysis.ternary_plots import TernaryAxesSubplotPoisson, ternary_figure
+from poisson_approval.meta_analysis.ternary_shortcuts import ternary_plot_n_equilibria, \
+    ternary_plot_winners_at_equilibrium, ternary_plot_winning_frequencies, SimplexToProfile
