@@ -7,6 +7,11 @@ from poisson_approval.strategies.StrategyOrdinal import StrategyOrdinal
 class GeneratorStrategyOrdinalUniform:
     """A generator of ordinal strategies (:class:`StrategyOrdinal`) following the uniform distribution.
 
+    Parameters
+    ----------
+    kwargs : keyword arguments
+        These additional arguments will be passed directly to :class:`StrategyOrdinal`.
+
     Notes
     -----
     Each ballot is chosen at random: either the top candidate or the two top candidates.
@@ -20,6 +25,9 @@ class GeneratorStrategyOrdinalUniform:
         <abc: ab, acb: ac, bac: b, bca: bc, cab: ac, cba: bc>
     """
 
+    def __init__(self, **kwargs):
+        self.kwargs = kwargs
+
     def __call__(self):
         """
         Returns
@@ -29,4 +37,4 @@ class GeneratorStrategyOrdinalUniform:
         """
         return StrategyOrdinal(d_ranking_ballot={
             ranking: random.choice([ballot_one(ranking), ballot_one_two(ranking)])
-            for ranking in RANKINGS})
+            for ranking in RANKINGS}, **self.kwargs)
