@@ -10,6 +10,8 @@ class GeneratorTauVectorGridUniform:
     ----------
     denominator : int
         The coefficients of the tau-vector will be fractions with this denominator.
+    kwargs : keyword arguments
+        These additional arguments will be passed directly to :class:`TauVector`.
 
     Notes
     -----
@@ -25,8 +27,9 @@ class GeneratorTauVectorGridUniform:
         <a: 1/50, ac: 11/50, b: 23/100, bc: 19/100, c: 17/50> ==> c
     """
 
-    def __init__(self, denominator):
+    def __init__(self, denominator, **kwargs):
         self.denominator = denominator
+        self.kwargs = kwargs
 
     def __call__(self):
         """
@@ -36,4 +39,4 @@ class GeneratorTauVectorGridUniform:
             A tau-vector.
         """
         x = rand_simplex_grid(d=6, denominator=self.denominator)
-        return TauVector({ballot: x[i] for i, ballot in enumerate(BALLOTS_WITHOUT_INVERSIONS)})
+        return TauVector({ballot: x[i] for i, ballot in enumerate(BALLOTS_WITHOUT_INVERSIONS)}, **self.kwargs)
