@@ -1,11 +1,19 @@
-from math import isclose
+import warnings
+import math
+import numpy as np
+from fractions import Fraction
+from poisson_approval.constants.constants import *
 from poisson_approval.constants.EquilibriumStatus import EquilibriumStatus
+from poisson_approval.generators.GeneratorStrategyThresholdUniform import GeneratorStrategyThresholdUniform
 from poisson_approval.profiles.Profile import Profile
 from poisson_approval.strategies.Strategy import Strategy
 from poisson_approval.strategies.StrategyThreshold import StrategyThreshold
 from poisson_approval.tau_vector.TauVector import TauVector
+from poisson_approval.utils.ComputationEngineNumeric import ComputationEngineNumeric
 from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
-from poisson_approval.utils.Util import *
+from poisson_approval.utils.Util import candidates_to_probabilities, my_division, array_to_d_candidate_value, \
+    one_over_t_plus_one, ballot_one, ballot_one_two, ballot_low_u, ballot_high_u, \
+    to_callable, candidates_to_d_candidate_probability
 from poisson_approval.utils.UtilCache import cached_property, property_deleting_cache
 
 
@@ -22,10 +30,12 @@ class ProfileCardinal(Profile):
         and `ratio_fanatic` must not exceed 1.
     voting_rule : str
         The voting rule. Possible values are ``APPROVAL``, ``PLURALITY`` and ``ANTI_PLURALITY``.
+    symbolic : bool
+        Whether the computations are symbolic or numeric.
     """
 
-    def __init__(self, ratio_sincere=0, ratio_fanatic=0, voting_rule=APPROVAL):
-        super().__init__(voting_rule=voting_rule)
+    def __init__(self, ratio_sincere=0, ratio_fanatic=0, voting_rule=APPROVAL, symbolic=False):
+        super().__init__(voting_rule=voting_rule, symbolic=symbolic)
         self.ratio_sincere = ratio_sincere
         self.ratio_fanatic = ratio_fanatic
 
@@ -46,7 +56,7 @@ class ProfileCardinal(Profile):
         -------
         Number
             The share of voters who have ranking `ranking` and a utility for their middle candidate strictly greater
-            than `u`.
+            than `u`. This does NOT include the voters who have a weak order of preference.
         """
         raise NotImplementedError
 
@@ -64,6 +74,8 @@ class ProfileCardinal(Profile):
         -------
         Number
             The share of voters who have ranking `ranking` and a utility for their middle candidate equal to `u`.
+            This does NOT include the voters who have a weak order of preference. I.e. if `u`=0 or `u=1`, then the
+            share is 0.
         """
         raise NotImplementedError
 
@@ -81,7 +93,7 @@ class ProfileCardinal(Profile):
         -------
         Number
             The share of voters who have ranking `ranking` and a utility for their middle candidate strictly lower
-            than `u`.
+            than `u`. This does NOT include the voters who have a weak order of preference.
         """
         raise NotImplementedError
 
@@ -95,6 +107,10 @@ class ProfileCardinal(Profile):
     def __eq__(self, other):
         raise NotImplementedError
 
+    @cached_property
+    def d_weak_order_share(self):
+        raise NotImplementedError
+
     @cached_property
     def standardized_version(self):
         raise NotImplementedError
@@ -120,11 +136,11 @@ class ProfileCardinal(Profile):
         tau_sincere = self.tau_sincere
         tau_fanatic = self.tau_fanatic
         tau_strategic = self.tau_strategic(strategy)
-        t = {ballot: barycenter(a=tau_strategic.d_ballot_share[ballot],
-                                b=[tau_sincere.d_ballot_share[ballot], tau_fanatic.d_ballot_share[ballot]],
-                                ratio_b=[self.ratio_sincere, self.ratio_fanatic])
+        t = {ballot: self.ce.barycenter(a=tau_strategic.d_ballot_share[ballot],
+                                        b=[tau_sincere.d_ballot_share[ballot], tau_fanatic.d_ballot_share[ballot]],
+                                        ratio_b=[self.ratio_sincere, self.ratio_fanatic])
              for ballot in BALLOTS_WITHOUT_INVERSIONS}
-        return TauVector(t, voting_rule=self.voting_rule)
+        return TauVector(t, voting_rule=self.voting_rule, symbolic=self.symbolic)
 
     @cached_property
     def tau_sincere(self):
@@ -142,7 +158,7 @@ class ProfileCardinal(Profile):
         -----
         In Plurality and Anti-plurality, sincere and fanatic voting are the same. They differ only in Approval.
         """
-        t = {ballot: 0 for ballot in BALLOTS_WITHOUT_INVERSIONS}
+        t = self.d_ballot_share_weak_voters_sincere.copy()
         for ranking in RANKINGS:
             if self.voting_rule == APPROVAL:
                 share_vote_one_two = self.have_ranking_with_utility_above_u(ranking, Fraction(1, 2))
@@ -155,7 +171,7 @@ class ProfileCardinal(Profile):
                 t[ballot_one_two(ranking)] += self.d_ranking_share[ranking]
             else:
                 raise NotImplementedError
-        return TauVector(t, voting_rule=self.voting_rule)
+        return TauVector(t, voting_rule=self.voting_rule, symbolic=self.symbolic)
 
     @cached_property
     def tau_fanatic(self):
@@ -171,7 +187,7 @@ class ProfileCardinal(Profile):
         -----
         In Plurality and Anti-plurality, sincere and fanatic voting are the same. They differ only in Approval.
         """
-        t = {ballot: 0 for ballot in BALLOTS_WITHOUT_INVERSIONS}
+        t = self.d_ballot_share_weak_voters_fanatic.copy()
         for ranking, share in self.d_ranking_share.items():
             if self.voting_rule in {APPROVAL, PLURALITY}:
                 t[ballot_one(ranking)] += share
@@ -179,7 +195,7 @@ class ProfileCardinal(Profile):
                 t[ballot_one_two(ranking)] += self.d_ranking_share[ranking]
             else:
                 raise NotImplementedError
-        return TauVector(t, voting_rule=self.voting_rule)
+        return TauVector(t, voting_rule=self.voting_rule, symbolic=self.symbolic)
 
     def tau_strategic(self, strategy):
         """Tau-vector associated to a strategy (fully strategic voting).
@@ -187,7 +203,9 @@ class ProfileCardinal(Profile):
         Parameters
         ----------
         strategy : StrategyThreshold
-            A strategy that specifies at least all the rankings that are present in the profile.
+            A strategy that specifies at least all the rankings that are present in the profile. If some voters
+            have a utility for their second candidate that is equal to the threshold utility of the strategy, then the
+            ratio of optimistic voters must be specified.
 
         Returns
         -------
@@ -195,15 +213,20 @@ class ProfileCardinal(Profile):
             Tau-vector associated to this profile and strategy `strategy`.
         """
         assert self.voting_rule == strategy.voting_rule
-        t = {ballot: 0 for ballot in BALLOTS_WITHOUT_INVERSIONS}
+        t = self.d_ballot_share_weak_voters_sincere.copy()  # For weak orders, strategic = sincere
         for ranking, threshold in strategy.d_ranking_threshold.items():
             if self.d_ranking_share[ranking] == 0:
                 continue
-            t[ballot_low_u(ranking, self.voting_rule)] += (
-                self.have_ranking_with_utility_u(ranking, u=threshold)
-                + self.have_ranking_with_utility_below_u(ranking, u=threshold))
+            t[ballot_low_u(ranking, self.voting_rule)] += self.have_ranking_with_utility_below_u(ranking, u=threshold)
             t[ballot_high_u(ranking, self.voting_rule)] += self.have_ranking_with_utility_above_u(ranking, u=threshold)
-        return TauVector(t, voting_rule=self.voting_rule)
+            share_limit_voters = self.have_ranking_with_utility_u(ranking, u=threshold)
+            if share_limit_voters != 0:
+                ratio_optimistic = strategy.d_ranking_ratio_optimistic[ranking]
+                t[ballot_low_u(ranking, self.voting_rule)] += self.ce.multiply_with_absorbing_zero(
+                    share_limit_voters, ratio_optimistic)
+                t[ballot_high_u(ranking, self.voting_rule)] += self.ce.multiply_with_absorbing_zero(
+                    share_limit_voters, 1 - ratio_optimistic)
+        return TauVector(t, voting_rule=self.voting_rule, symbolic=self.symbolic)
 
     def is_equilibrium(self, strategy):
         """Whether a strategy is an equilibrium.
@@ -211,7 +234,9 @@ class ProfileCardinal(Profile):
         Parameters
         ----------
         strategy : StrategyThreshold
-            A strategy that specifies at least all the rankings that are present in the profile.
+            A strategy that specifies at least all the rankings that are present in the profile. If some voters
+            have a utility for their second candidate that is equal to the threshold utility of the strategy, then the
+            ratio of optimistic voters must be specified.
 
         Returns
         -------
@@ -225,19 +250,26 @@ class ProfileCardinal(Profile):
         """
         this_tau = self.tau(strategy)
         d_ranking_best_response = this_tau.d_ranking_best_response
-        d_ranking_threshold = dict()
         for ranking, share in self.d_ranking_share.items():
             if share == 0:
                 continue
-            best_response = d_ranking_best_response[ranking]
-            if best_response.ballot == INCONCLUSIVE:  # pragma: no cover
-                return EquilibriumStatus.INCONCLUSIVE
-            d_ranking_threshold[ranking] = best_response.threshold_utility
-        tau_response = self.tau(StrategyThreshold(d_ranking_threshold))
-        if tau_response.isclose(this_tau):
-            return EquilibriumStatus.EQUILIBRIUM
-        else:
-            return EquilibriumStatus.NOT_EQUILIBRIUM
+            actual_threshold = strategy.d_ranking_threshold[ranking]
+            br_threshold = d_ranking_best_response[ranking].threshold_utility
+            test = (self.ce.look_equal(self.have_ranking_with_utility_below_u(ranking, actual_threshold),
+                                       self.have_ranking_with_utility_below_u(ranking, br_threshold), abs_tol=1E-9)
+                    and self.ce.look_equal(self.have_ranking_with_utility_above_u(ranking, actual_threshold),
+                                           self.have_ranking_with_utility_above_u(ranking, br_threshold), abs_tol=1E-9))
+            if not test:
+                return EquilibriumStatus.NOT_EQUILIBRIUM
+        return EquilibriumStatus.EQUILIBRIUM
+
+    @property
+    def strategies_pure(self):
+        raise NotImplementedError
+
+    @property
+    def strategies_group(self):
+        raise NotImplementedError
 
     def _initializer(self, init):
         """Initial condition for iterated voting or fictitious play.
@@ -247,7 +279,8 @@ class ProfileCardinal(Profile):
         init : Strategy or TauVector or str
             If it is a strategy, it must be an argument accepted by :meth:`tau`, i.e. by :meth:`tau_strategic`.
             If it is a tau-vector, it is used directly. If it is the string ``'sincere'`` or ``'fanatic'``,
-            then :attr:`tau_sincere` or :attr:`tau_fanatic` is respectively used.
+            then :attr:`tau_sincere` or :attr:`tau_fanatic` is respectively used. If it is the string ``'random'``,
+            then :meth:`random_strategy` is used.
 
         Returns
         -------
@@ -256,9 +289,15 @@ class ProfileCardinal(Profile):
         tau : TauVector
             The initial tau-vector.
         """
+        if self.symbolic:  # pragma: no cover
+            warnings.warn('Using fictitious play or iterated voting with symbolic=True is strongly discouraged. '
+                          'Consider defining the profile with symbolic=False.')
         if isinstance(init, Strategy):
             strategy = init.deepcopy_with_attached_profile(self)
             tau = strategy.tau
+        elif init == 'random':
+            strategy = self.random_strategy(profile=self)
+            tau = strategy.tau
         else:
             strategy = None
             if isinstance(init, TauVector):
@@ -267,12 +306,13 @@ class ProfileCardinal(Profile):
                 tau = self.tau_sincere
             elif init == 'fanatic':
                 tau = self.tau_fanatic
-            else:
+            else:  # pragma: no cover
                 raise ValueError
         return strategy, tau
 
     def iterated_voting(self, init, n_max_episodes,
-                        perception_update_ratio=1, ballot_update_ratio=1, verbose=False):
+                        perception_update_ratio=1, ballot_update_ratio=1,
+                        winning_frequency_update_ratio=one_over_t_plus_one, verbose=False):
         """Seek for convergence by iterated voting.
 
         Parameters
@@ -289,6 +329,13 @@ class ProfileCardinal(Profile):
         ballot_update_ratio : Number in [0, 1]
             The ratio of voters who update their ballot:
             ``tau_actual = (1 - ballot_update_ratio) * tau_actual + ballot_update_ratio * tau_response``.
+        winning_frequency_update_ratio : callable or Number
+            The coefficient when updating the winning frequency of each candidate:
+            ``d_candidate_winning_frequency[c] =
+            (1 - winning_frequency_update_ratio(t)) * d_candidate_winning_frequency[c]
+            + winning_frequency_update_ratio(t) * winning_probability[c]``.
+            The default function is :func:`~utils.Util.one_over_t_plus_one`, which leads to an arithmetic average.
+            Note that this parameters has an influence only in case of non-convergence.
         verbose : bool
             If True, print all intermediate steps.
 
@@ -325,11 +372,13 @@ class ProfileCardinal(Profile):
         In general, you should use :meth:`iterated_voting` only if you care about cycles, with the constraint
         that it implies having constant update ratios.
         """
+        winning_frequency_update_ratio = to_callable(winning_frequency_update_ratio)
         strategy, tau_actual = self._initializer(init)
         tau_perceived = tau_actual
         strategies = [strategy]
         taus_actual = [tau_actual]
         taus_perceived = [tau_perceived]
+        array_candidate_winning_frequency = candidates_to_probabilities(tau_actual.winners)
         if verbose:
             print('t = %s' % 0)
             print('strategy: %s' % strategy)
@@ -337,26 +386,29 @@ class ProfileCardinal(Profile):
         n_episodes = n_max_episodes
         for t in range(1, n_max_episodes + 1):
             tau_perceived = TauVector({
-                ballot: _my_round(barycenter(a=tau_perceived.d_ballot_share[ballot],
-                                             b=tau_actual.d_ballot_share[ballot],
-                                             ratio_b=perception_update_ratio))
+                ballot: _my_round(ComputationEngineNumeric.barycenter(a=tau_perceived.d_ballot_share[ballot],
+                                                                      b=tau_actual.d_ballot_share[ballot],
+                                                                      ratio_b=perception_update_ratio))
                 for ballot in BALLOTS_WITHOUT_INVERSIONS
-            }, voting_rule=self.voting_rule)
+            }, voting_rule=self.voting_rule, symbolic=self.symbolic)
             strategy = self.best_responses_to_strategy(tau_perceived.d_ranking_best_response)
             tau_full_response = strategy.tau
             tau_actual = TauVector({
-                ballot: _my_round(barycenter(a=tau_actual.d_ballot_share[ballot],
-                                             b=tau_full_response.d_ballot_share[ballot],
-                                             ratio_b=ballot_update_ratio))
+                ballot: _my_round(ComputationEngineNumeric.barycenter(a=tau_actual.d_ballot_share[ballot],
+                                                                      b=tau_full_response.d_ballot_share[ballot],
+                                                                      ratio_b=ballot_update_ratio))
                 for ballot in BALLOTS_WITHOUT_INVERSIONS
-            }, normalization_warning=False, voting_rule=self.voting_rule)
+            }, normalization_warning=False, voting_rule=self.voting_rule, symbolic=self.symbolic)
+            array_candidate_winning_frequency = (
+                (1 - winning_frequency_update_ratio(t)) * array_candidate_winning_frequency
+                + winning_frequency_update_ratio(t) * candidates_to_probabilities(tau_actual.winners))
             if verbose:
                 print('t = %s' % t)
                 print('tau_perceived: %s' % tau_perceived)
                 print('strategy: %s' % strategy)
                 print('tau_full_response: %s' % tau_full_response)
                 print('tau_actual: %s' % tau_actual)
-            # If there is an exact cycle, it is useless to continue looping.
+            # If there is an symbolic cycle, it is useless to continue looping.
             break_ = (tau_actual, tau_perceived) in zip(taus_actual, taus_perceived)
 
             taus_actual.append(tau_actual)
@@ -379,7 +431,7 @@ class ProfileCardinal(Profile):
             cycle_taus_actual = []
             cycle_taus_perceived = []
             cycle_strategies = []
-            d_candidate_winning_frequency = _d_candidate_winning_frequency(taus_actual)
+            d_candidate_winning_frequency = array_to_d_candidate_value(array_candidate_winning_frequency)
         return {'cycle_taus_perceived': cycle_taus_perceived,
                 'cycle_strategies': cycle_strategies,
                 'cycle_taus_actual': cycle_taus_actual,
@@ -387,7 +439,8 @@ class ProfileCardinal(Profile):
                 'd_candidate_winning_frequency': d_candidate_winning_frequency}
 
     def fictitious_play(self, init, n_max_episodes,
-                        perception_update_ratio=one_over_t_plus_one, ballot_update_ratio=1, verbose=False):
+                        perception_update_ratio=one_over_t_plus_one, ballot_update_ratio=1,
+                        winning_frequency_update_ratio=one_over_t_plus_one, verbose=False):
         """Seek for convergence by fictitious play.
 
         Parameters
@@ -411,6 +464,13 @@ class ProfileCardinal(Profile):
             For any ``t`` from 1 to `n_max_episodes` included, the update ratio must be in [0, 1]. The default function
             is the constant 1, which corresponds to a full update. If `ballot_update_ratio` is a Number, it is
             considered as a constant function.
+        winning_frequency_update_ratio : callable or Number
+            The coefficient when updating the winning frequency of each candidate:
+            ``d_candidate_winning_frequency[c] =
+            (1 - winning_frequency_update_ratio(t)) * d_candidate_winning_frequency[c]
+            + winning_frequency_update_ratio(t) * winning_probability[c]``.
+            The default function is :func:`~utils.Util.one_over_t_plus_one`, which leads to an arithmetic average.
+            Note that this parameters has an influence only in case of non-convergence.
         verbose : bool
             If True, print all intermediate steps.
 
@@ -441,9 +501,10 @@ class ProfileCardinal(Profile):
         """
         perception_update_ratio = to_callable(perception_update_ratio)
         ballot_update_ratio = to_callable(ballot_update_ratio)
+        winning_frequency_update_ratio = to_callable(winning_frequency_update_ratio)
 
         strategy, tau_actual = self._initializer(init)
-        array_candidate_winning_count = candidates_to_probabilities(tau_actual.winners)
+        array_candidate_winning_frequency = candidates_to_probabilities(tau_actual.winners)
         tau_perceived = tau_actual
         if verbose:
             print('t = %s' % 0)
@@ -453,20 +514,22 @@ class ProfileCardinal(Profile):
 
         for t in range(1, n_max_episodes + 1):
             tau_perceived = TauVector({
-                ballot: _my_round(barycenter(a=tau_perceived.d_ballot_share[ballot],
-                                             b=tau_actual.d_ballot_share[ballot],
-                                             ratio_b=perception_update_ratio(t)))
+                ballot: _my_round(ComputationEngineNumeric.barycenter(a=tau_perceived.d_ballot_share[ballot],
+                                                                      b=tau_actual.d_ballot_share[ballot],
+                                                                      ratio_b=perception_update_ratio(t)))
                 for ballot in BALLOTS_WITHOUT_INVERSIONS
-            }, voting_rule=self.voting_rule)
+            }, voting_rule=self.voting_rule, symbolic=self.symbolic)
             strategy = self.best_responses_to_strategy(tau_perceived.d_ranking_best_response)
             tau_full_response = strategy.tau
             tau_actual = TauVector({
-                ballot: _my_round(barycenter(a=tau_actual.d_ballot_share[ballot],
-                                             b=tau_full_response.d_ballot_share[ballot],
-                                             ratio_b=ballot_update_ratio(t)))
+                ballot: _my_round(ComputationEngineNumeric.barycenter(a=tau_actual.d_ballot_share[ballot],
+                                                                      b=tau_full_response.d_ballot_share[ballot],
+                                                                      ratio_b=ballot_update_ratio(t)))
                 for ballot in BALLOTS_WITHOUT_INVERSIONS
-            }, normalization_warning=False, voting_rule=self.voting_rule)
-            array_candidate_winning_count += candidates_to_probabilities(tau_actual.winners)
+            }, normalization_warning=False, voting_rule=self.voting_rule, symbolic=self.symbolic)
+            array_candidate_winning_frequency = (
+                (1 - winning_frequency_update_ratio(t)) * array_candidate_winning_frequency
+                + winning_frequency_update_ratio(t) * candidates_to_probabilities(tau_actual.winners))
             if verbose:
                 print('t = %s' % t)
                 print('tau_perceived: %s' % tau_perceived)
@@ -478,11 +541,28 @@ class ProfileCardinal(Profile):
                 return {'tau': tau_full_response, 'strategy': strategy, 'n_episodes': t,
                         'd_candidate_winning_frequency': candidates_to_d_candidate_probability(tau_actual.winners)}
         n_taus = n_max_episodes + 1
-        array_candidate_winning_frequency = array_candidate_winning_count / n_taus
         d_candidate_winning_frequency = array_to_d_candidate_value(array_candidate_winning_frequency)
         return {'tau': None, 'strategy': None, 'n_episodes': n_max_episodes,
                 'd_candidate_winning_frequency': d_candidate_winning_frequency}
 
+    @classmethod
+    def order_and_label(cls, t):
+        raise NotImplementedError
+
+    @classmethod
+    def random_strategy(cls, **kwargs):
+        """Random strategy.
+
+        This is a default generator of random strategies. It is used, for example, in
+        :class:`ProfileCardinal.iterated_voting`.
+
+        Returns
+        -------
+        StrategyThreshold
+            Uses :class:`GeneratorStrategyThresholdUniform`.
+        """
+        return GeneratorStrategyThresholdUniform(**kwargs)()
+
 
 def _my_round(x):
     """Round to 0 or 1 if the number is close enough.
@@ -494,7 +574,7 @@ def _my_round(x):
     Returns
     -------
     Number
-        Generally, return `x`. If `x` is close enough to 0 or 1, then return 0 or 1 respectively.
+        Generally, return `x` (converted to float). If `x` is close enough to 0 or 1, then return 0 or 1 respectively.
 
     Examples
     --------
@@ -506,11 +586,11 @@ def _my_round(x):
         >>> _my_round(0.123456789)
         0.123456789
     """
-    if isclose(x, 1):
+    if math.isclose(x, 1):
         return 1
-    if isclose(x, 0, abs_tol=1E-9):
+    if math.isclose(x, 0, abs_tol=1E-9):
         return 0
-    return x
+    return float(x)
 
 
 def _d_candidate_winning_frequency(taus):
@@ -529,6 +609,7 @@ def _d_candidate_winning_frequency(taus):
 
     Examples
     --------
+        >>> from fractions import Fraction
         >>> tau_1 = TauVector({'a': Fraction(2, 5), 'b': Fraction(2, 5), 'c': Fraction(1, 5)})
         >>> tau_2 = TauVector({'a': 0, 'b': 0, 'c': 1})
         >>> _d_candidate_winning_frequency([tau_1, tau_2])
@@ -538,10 +619,5 @@ def _d_candidate_winning_frequency(taus):
     for tau in taus:
         counts = counts + candidates_to_probabilities(tau.winners)
     n_taus = len(taus)
-    frequencies = np.array([_my_division(count, n_taus) for count in counts])
+    frequencies = np.array([my_division(count, n_taus) for count in counts])
     return array_to_d_candidate_value(frequencies)
-
-
-def _my_division(x, y):
-    result = Fraction(x, y)
-    return result.numerator if result.denominator == 1 else result
