@@ -1,16 +1,15 @@
 import warnings
+import itertools
 import numpy as np
-from math import isclose
 from poisson_approval.constants.constants import *
 from poisson_approval.constants.EquilibriumStatus import EquilibriumStatus
-from poisson_approval.containers.AnalyzedStrategies import AnalyzedStrategies
-from poisson_approval.containers.Winners import Winners
+from poisson_approval.generators.GeneratorStrategyOrdinalUniform import GeneratorStrategyOrdinalUniform
 from poisson_approval.profiles.Profile import Profile
 from poisson_approval.strategies.StrategyOrdinal import StrategyOrdinal
 from poisson_approval.tau_vector.TauVector import TauVector
 from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
-from poisson_approval.utils.Util import ballot_one, ballot_two, ballot_one_two, ballot_one_three, barycenter, \
-    product_dict, ballot_low_u, ballot_high_u
+from poisson_approval.utils.Util import ballot_one, ballot_two, ballot_one_two, ballot_one_three, \
+    product_dict, ballot_low_u, ballot_high_u, sort_weak_order, my_division
 from poisson_approval.utils.UtilCache import cached_property, property_deleting_cache
 from poisson_approval.utils.UtilMasks import masks_area, masks_distribution, winners_distribution
 
@@ -22,8 +21,11 @@ class ProfileOrdinal(Profile):
     Parameters
     ----------
     d_ranking_share : dict
-        E.g. ``{'abc': 0.4, 'cab': 0.6}``. ``d_ranking_share['abc']`` is the probability that a voter prefers
+        E.g. ``{'abc': 0.4, 'cab': 0.3}``. ``d_ranking_share['abc']`` is the probability that a voter prefers
         candidate ``a``, then candidate ``b``, then candidate ``c``.
+    d_weak_order_share : dict
+        E.g. ``{'a~b>c': 0.2, 'a>b~c': 0.1}``. ``d_weak_order_share['a~b>c']`` is the probability that a voter likes
+        candidates ``a`` and ``b`` equally and prefer them to candidate ``c``.
     normalization_warning : bool
         Whether a warning should be issued if the input distribution is not normalized.
     well_informed_voters : bool.
@@ -34,6 +36,8 @@ class ProfileOrdinal(Profile):
         The ratio of fanatic voters, in the interval [0, 1]. This is used for :meth:`tau`.
     voting_rule : str
         The voting rule. Possible values are ``APPROVAL``, ``PLURALITY`` and ``ANTI_PLURALITY``.
+    symbolic : bool
+        Whether the computations are symbolic or numeric.
 
     Notes
     -----
@@ -70,22 +74,60 @@ class ProfileOrdinal(Profile):
         {'abc', 'bac', 'cab'}
         >>> profile.is_generic_in_rankings  # Are all rankings there?
         False
+        >>> profile.analyzed_strategies_ordinal
+        Equilibria:
+        <abc: a, bac: b, cab: ac> ==> b (FF)
+        <abc: a, bac: ab, cab: c> ==> a (D)
+        <BLANKLINE>
+        Utility-dependent equilibrium:
+        <abc: ab, bac: b, cab: c> ==> b (FF)
+        <BLANKLINE>
+        Non-equilibria:
+        <abc: a, bac: b, cab: c> ==> b (FF)
+        <abc: a, bac: ab, cab: ac> ==> a (D)
+        <abc: ab, bac: b, cab: ac> ==> b (FF)
+        <abc: ab, bac: ab, cab: c> ==> a, b (FF)
+        <abc: ab, bac: ab, cab: ac> ==> a (D)
+        >>> print(profile.analyzed_strategies_ordinal.equilibria[0])
+        <abc: a, bac: b, cab: ac> ==> b
+        >>> print(profile.analyzed_strategies_ordinal.winners_at_equilibrium)
+        a, b
+
+    The profile can include weak orders:
+
+        >>> from fractions import Fraction
+        >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10)},
+        ...                          d_weak_order_share={'c>a~b': Fraction(3, 10)})
+        >>> profile
+        ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(3, 5)}, d_weak_order_share={'c>a~b': Fraction(3, 10)})
+        >>> print(profile)
+        <abc: 1/10, bac: 3/5, c>a~b: 3/10> (Condorcet winner: b)
     """
 
-    def __init__(self, d_ranking_share, normalization_warning=True, well_informed_voters=True, ratio_fanatic=0,
-                 voting_rule=APPROVAL):
-        super().__init__(voting_rule=voting_rule)
-        # Populate the dictionary
-        self._d_ranking_share = DictPrintingInOrderIgnoringZeros({ranking: 0 for ranking in RANKINGS})
-        for ranking, share in d_ranking_share.items():
-            self._d_ranking_share[ranking] += share
+    def __init__(self, d_ranking_share, d_weak_order_share=None, normalization_warning=True, well_informed_voters=True,
+                 ratio_fanatic=0, voting_rule=APPROVAL, symbolic=False):
+        super().__init__(voting_rule=voting_rule, symbolic=symbolic)
+        if d_weak_order_share is None:
+            d_weak_order_share = dict()
+        # Populate the dictionaries of rankings and weak orders
+        self._d_ranking_share = DictPrintingInOrderIgnoringZeros({
+            ranking: 0 for ranking in RANKINGS})
+        self._d_weak_order_share = DictPrintingInOrderIgnoringZeros({
+            weak_order: 0 for weak_order in WEAK_ORDERS_WITHOUT_INVERSIONS})
+        for order, share in itertools.chain(d_ranking_share.items(), d_weak_order_share.items()):
+            try:
+                self._d_ranking_share[order] += share
+            except KeyError:
+                self._d_weak_order_share[sort_weak_order(order)] += share
         # Normalize if necessary
-        total = sum(self._d_ranking_share.values())
-        if not isclose(total, 1.):
+        total = sum(self._d_ranking_share.values()) + sum(self._d_weak_order_share.values())
+        if not self.ce.look_equal(total, 1):
             if normalization_warning:
-                warnings.warn("Warning: profile is not normalized, I will normalize it.")
+                warnings.warn(NORMALIZATION_WARNING)
             for ranking in self._d_ranking_share.keys():
-                self._d_ranking_share[ranking] /= total
+                self._d_ranking_share[ranking] = my_division(self._d_ranking_share[ranking], total)
+            for weak_order in self._d_weak_order_share.keys():
+                self._d_weak_order_share[weak_order] = my_division(self._d_weak_order_share[weak_order], total)
         # Other parameters
         self.well_informed_voters = well_informed_voters
         self.ratio_fanatic = ratio_fanatic
@@ -97,6 +139,10 @@ class ProfileOrdinal(Profile):
     def d_ranking_share(self):
         return self._d_ranking_share
 
+    @cached_property
+    def d_weak_order_share(self):
+        return self._d_weak_order_share
+
     def __repr__(self):
         """
             >>> from fractions import Fraction
@@ -106,7 +152,9 @@ class ProfileOrdinal(Profile):
             ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(3, 5), 'cab': Fraction(3, 10)}, \
 well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
         """
-        arguments = repr(self._d_ranking_share)
+        arguments = repr(self.d_ranking_share)
+        if self.contains_weak_orders:
+            arguments += ', d_weak_order_share=%r' % self.d_weak_order_share
         if not self.well_informed_voters:
             arguments += ', well_informed_voters=False'
         if self.ratio_fanatic > 0:
@@ -123,7 +171,12 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
             >>> print(profile)
             <abc: 1/10, bac: 3/5, cab: 3/10> (Condorcet winner: b) (badly informed voters) (ratio_fanatic: 1/10)
         """
-        result = '<%s>' % str(self._d_ranking_share)[1:-1]
+        contents = []
+        if self.contains_rankings:
+            contents.append(str(self.d_ranking_share)[1:-1])
+        if self.contains_weak_orders:
+            contents.append(str(self.d_weak_order_share)[1:-1])
+        result = '<' + ', '.join(contents) + '>'
         if self.is_profile_condorcet:
             result += ' (Condorcet winner: %s)' % self.condorcet_winners
         if not self.well_informed_voters:
@@ -159,6 +212,7 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
         """
         return (isinstance(other, ProfileOrdinal)
                 and self.d_ranking_share == other.d_ranking_share
+                and self.d_weak_order_share == other.d_weak_order_share
                 and self.well_informed_voters == other.well_informed_voters
                 and self.ratio_fanatic == other.ratio_fanatic
                 and self.voting_rule == other.voting_rule)
@@ -183,41 +237,19 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
         best_signature = []
         for perm in XYZ_PERMUTATIONS:
             d_test = {translate(ranking, perm): share for ranking, share in self.d_ranking_share.items()}
+            d_test.update({sort_weak_order(translate(weak_order, perm)): share
+                           for weak_order, share in self.d_weak_order_share.items()})
             signature_test = [d_test[ranking] for ranking in XYZ_RANKINGS]
+            signature_test += [d_test[weak_order] for weak_order in XYZ_WEAK_ORDERS_WITHOUT_INVERSIONS]
             if signature_test > best_signature:
                 best_signature = signature_test
                 best_d = d_test
         return ProfileOrdinal({ranking: best_d[xyz_ranking] for ranking, xyz_ranking in zip(RANKINGS, XYZ_RANKINGS)},
+                              {weak_order: best_d[xyz_weak_order] for weak_order, xyz_weak_order in zip(
+                                  WEAK_ORDERS_WITHOUT_INVERSIONS, XYZ_WEAK_ORDERS_WITHOUT_INVERSIONS)},
                               well_informed_voters=self.well_informed_voters, ratio_fanatic=self.ratio_fanatic,
                               voting_rule=self.voting_rule)
 
-    # Has full support
-    @cached_property
-    def support(self):
-        """:class:`SetPrintingInOrder` of str : Alternate notation for :meth:`support_in_rankings`.
-
-        Examples
-        --------
-            >>> from fractions import Fraction
-            >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)})
-            >>> profile.support
-            {'abc', 'bac', 'cab'}
-        """
-        return self.support_in_rankings
-
-    @cached_property
-    def is_generic(self):
-        """bool : Alternate notation for :meth:`is_generic_in_rankings`
-
-        Examples
-        --------
-            >>> from fractions import Fraction
-            >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)})
-            >>> profile.is_generic
-            False
-        """
-        return self.is_generic_in_rankings
-
     # Tau and strategy-related stuff
 
     def tau(self, strategy):
@@ -249,11 +281,11 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
         """
         tau_fanatic = self.tau_fanatic
         tau_strategic = self.tau_strategic(strategy)
-        t = {ballot: barycenter(a=tau_strategic.d_ballot_share[ballot],
-                                b=tau_fanatic.d_ballot_share[ballot],
-                                ratio_b=self.ratio_fanatic)
+        t = {ballot: self.ce.barycenter(a=tau_strategic.d_ballot_share[ballot],
+                                        b=tau_fanatic.d_ballot_share[ballot],
+                                        ratio_b=self.ratio_fanatic)
              for ballot in BALLOTS_WITHOUT_INVERSIONS}
-        return TauVector(t, voting_rule=self.voting_rule)
+        return TauVector(t, voting_rule=self.voting_rule, symbolic=self.symbolic)
 
     @cached_property
     def tau_fanatic(self):
@@ -265,7 +297,7 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
             In Approval or Plurality, all voters approve of their top candidate only. In Anti-Plurality, they all
             disapprove of their bottom candidate, i.e. they approve their two first candidates.
         """
-        t = {ballot: 0 for ballot in BALLOTS_WITHOUT_INVERSIONS}
+        t = self.d_ballot_share_weak_voters_fanatic.copy()
         if self.voting_rule in {APPROVAL, PLURALITY}:
             for ranking, share in self.d_ranking_share.items():
                 t[ballot_one(ranking)] += share
@@ -274,7 +306,7 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
                 t[ballot_one_two(ranking)] += share
         else:
             raise NotImplementedError
-        return TauVector(t, voting_rule=self.voting_rule)
+        return TauVector(t, voting_rule=self.voting_rule, symbolic=self.symbolic)
 
     def tau_strategic(self, strategy):
         """Tau-vector associated to a strategy.
@@ -310,16 +342,16 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
             <a: 7/16, b: 3/8, c: 3/16> ==> a
         """
         assert self.voting_rule == strategy.voting_rule
-        t = {ballot: 0 for ballot in BALLOTS_WITHOUT_INVERSIONS}
+        t = self.d_ballot_share_weak_voters_sincere.copy()  # For weak orders, strategic = sincere
         for ranking, ballot in strategy.d_ranking_ballot.items():
             if self.d_ranking_share[ranking] > 0:
                 t[ballot] += self.d_ranking_share[ranking]
         if self.voting_rule == APPROVAL and not self.well_informed_voters:
             return TauVector(
                 {'a': t['a'] + t['ab'] + t['ac'], 'b': t['b'] + t['ab'] + t['bc'], 'c': t['c'] + t['ac'] + t['bc']},
-                normalization_warning=False, voting_rule=self.voting_rule)
+                normalization_warning=False, voting_rule=self.voting_rule, symbolic=self.symbolic)
         else:
-            return TauVector(t, voting_rule=self.voting_rule)
+            return TauVector(t, voting_rule=self.voting_rule, symbolic=self.symbolic)
 
     def is_equilibrium(self, strategy):
         """Whether a strategy is an equilibrium.
@@ -356,56 +388,6 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
                 return EquilibriumStatus.NOT_EQUILIBRIUM
         return status
 
-    @cached_property
-    def analyzed_strategies(self):
-        """AnalyzedStrategies : Analyzed strategies of the profile.
-
-        Examples
-        --------
-            >>> from fractions import Fraction
-            >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)})
-            >>> profile.analyzed_strategies
-            Equilibria:
-            <abc: a, bac: b, cab: ac> ==> b (FF)
-            <abc: a, bac: ab, cab: c> ==> a (D)
-            <BLANKLINE>
-            Utility-dependent equilibrium:
-            <abc: ab, bac: b, cab: c> ==> b (FF)
-            <BLANKLINE>
-            Non-equilibria:
-            <abc: a, bac: b, cab: c> ==> b (FF)
-            <abc: a, bac: ab, cab: ac> ==> a (D)
-            <abc: ab, bac: b, cab: ac> ==> b (FF)
-            <abc: ab, bac: ab, cab: c> ==> a, b (FF)
-            <abc: ab, bac: ab, cab: ac> ==> a (D)
-            >>> print(profile.analyzed_strategies.equilibria[0])
-            <abc: a, bac: b, cab: ac> ==> b
-            >>> print(profile.winners_at_equilibrium)
-            a, b
-        """
-        equilibria = []
-        utility_dependent = []
-        inconclusive = []
-        non_equilibria = []
-        d_ranking_possible_ballots = {ranking: [ballot_low_u(ranking, self.voting_rule),
-                                                ballot_high_u(ranking, self.voting_rule)]
-                                      if self.d_ranking_share[ranking] > 0 else ['']
-                                      for ranking in RANKINGS}
-        for d_ranking_ballot in product_dict(d_ranking_possible_ballots):
-            strategy = StrategyOrdinal(d_ranking_ballot, profile=self, voting_rule=self.voting_rule)
-            status = strategy.is_equilibrium
-            if status == EquilibriumStatus.EQUILIBRIUM:
-                equilibria.append(strategy)
-            elif status == EquilibriumStatus.UTILITY_DEPENDENT:
-                utility_dependent.append(strategy)
-            elif status == EquilibriumStatus.INCONCLUSIVE:  # pragma: no cover
-                inconclusive.append(strategy)
-                warnings.warn('Met an inconclusive case: \nprofile = %r\nstrategy = %r'
-                              % (self, strategy))
-            else:
-                non_equilibria.append(strategy)
-        return AnalyzedStrategies(equilibria, utility_dependent, inconclusive, non_equilibria)
-
     def proba_equilibrium(self, test=None):
         """Probability that an equilibrium exists (depending on the utilities).
 
@@ -434,16 +416,16 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
             # noinspection PyUnusedLocal
             def test(strategy):
                 return True
-        if any([test(strategy) for strategy in self.analyzed_strategies.equilibria]):
+        if any([test(strategy) for strategy in self.analyzed_strategies_ordinal.equilibria]):
             return 1
-        support = sorted(self.support)
+        support = sorted(self.support_in_rankings)
         dim = len(support)
         masks = [
             [(strategy.d_ranking_best_response[ranking].threshold_utility, len(strategy.d_ranking_ballot[ranking]) == 2)
              for ranking in support]
-            for strategy in self.analyzed_strategies.utility_dependent if test(strategy)
+            for strategy in self.analyzed_strategies_ordinal.utility_dependent if test(strategy)
         ]
-        return masks_area(inf=[0] * dim, sup=[1] * dim, masks=masks)
+        return self.ce.simplify(masks_area(inf=self.ce.zeros(dim), sup=self.ce.ones(dim), masks=masks))
 
     def distribution_equilibria(self, test=None):
         """Distribution of numbers of equilibria (depending on the utilities).
@@ -473,15 +455,17 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
             # noinspection PyUnusedLocal
             def test(strategy):
                 return True
-        cover_alls = np.sum([test(strategy) for strategy in self.analyzed_strategies.equilibria], dtype=int)
-        support = sorted(self.support)
+        cover_alls = np.sum([test(strategy) for strategy in self.analyzed_strategies_ordinal.equilibria], dtype=int)
+        support = sorted(self.support_in_rankings)
         dim = len(support)
         masks = [
             [(strategy.d_ranking_best_response[ranking].threshold_utility, len(strategy.d_ranking_ballot[ranking]) == 2)
              for ranking in support]
-            for strategy in self.analyzed_strategies.utility_dependent if test(strategy)
+            for strategy in self.analyzed_strategies_ordinal.utility_dependent if test(strategy)
         ]
-        return masks_distribution(inf=np.zeros(dim), sup=np.ones(dim), masks=masks, cover_alls=cover_alls)
+        return self.ce.simplify_vector(masks_distribution(inf=self.ce.zeros(dim),
+                                                          sup=self.ce.ones(dim),
+                                                          masks=masks, cover_alls=cover_alls))
 
     def distribution_winners(self, test=None):
         """Distribution of the number of equilibrium winners (depending on the utilities).
@@ -506,17 +490,17 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
             >>> from fractions import Fraction
             >>> profile = ProfileOrdinal({'abc': Fraction(1, 10), 'bac': Fraction(6, 10), 'cab': Fraction(3, 10)})
             >>> profile.distribution_winners()
-            array([0., 0., 1., 0.])
+            array([0, 0, 1, 0])
         """
         if test is None:
             # noinspection PyUnusedLocal
             def test(strategy):
                 return True
         cover_alls = set.union(*([set()] + [
-            strategy.winners for strategy in self.analyzed_strategies.equilibria
+            strategy.winners for strategy in self.analyzed_strategies_ordinal.equilibria
             if test(strategy)
         ]))
-        support = sorted(self.support)
+        support = sorted(self.support_in_rankings)
         dim = len(support)
         masks_winners = [
             (
@@ -525,7 +509,49 @@ well_informed_voters=False, ratio_fanatic=Fraction(1, 10))
                  for ranking in support],
                 strategy.winners
             )
-            for strategy in self.analyzed_strategies.utility_dependent if test(strategy)
+            for strategy in self.analyzed_strategies_ordinal.utility_dependent if test(strategy)
         ]
-        return winners_distribution(inf=np.zeros(dim), sup=np.ones(dim), masks_winners=masks_winners,
-                                    cover_alls=cover_alls)
+        return self.ce.simplify_vector(winners_distribution(inf=self.ce.zeros(dim),
+                                                            sup=self.ce.ones(dim),
+                                                            masks_winners=masks_winners,
+                                                            cover_alls=cover_alls))
+
+    @property
+    def strategies_pure(self):
+        raise NotImplementedError
+
+    @property
+    def strategies_group(self):
+        raise NotImplementedError
+
+    @classmethod
+    def order_and_label(cls, t):
+        r"""Order and label of a discrete type.
+
+        Cf. :meth:`Profile.order_and_label`.
+
+        Examples
+        --------
+            >>> ProfileOrdinal.order_and_label('abc')
+            ('abc', '$r(abc)$')
+            >>> ProfileOrdinal.order_and_label('a~b>c')
+            ('a~b>c', '$r(a\\sim b>c)$')
+        """
+        if len(t) == 3:
+            return t, '$r(%s)$' % t
+        else:
+            return cls.order_and_label_weak(t)
+
+    @classmethod
+    def random_strategy(cls, **kwargs):
+        """Random strategy.
+
+        This is a default generator of random strategies. It is used, for example, in
+        :class:`ProfileCardinal.iterated_voting`.
+
+        Returns
+        -------
+        StrategyOrdinal
+            Uses :class:`GeneratorStrategyOrdinalUniform`.
+        """
+        return GeneratorStrategyOrdinalUniform(**kwargs)()
