@@ -1,15 +1,16 @@
 import warnings
-from math import isclose
+import itertools
+from fractions import Fraction
 from poisson_approval.constants.constants import *
-from poisson_approval.utils.Util import ballot_one, ballot_two, ballot_one_two, ballot_one_three, sort_ballot, \
-    product_dict, ballot_low_u, ballot_high_u
-from poisson_approval.utils.SetPrintingInOrder import SetPrintingInOrder
-from poisson_approval.profiles.ProfileCardinal import ProfileCardinal
-from poisson_approval.tau_vector.TauVector import TauVector
-from poisson_approval.strategies.StrategyTwelve import StrategyTwelve
-from poisson_approval.containers.AnalyzedStrategies import AnalyzedStrategies
-from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
 from poisson_approval.constants.EquilibriumStatus import EquilibriumStatus
+from poisson_approval.generators.GeneratorStrategyTwelveUniform import GeneratorStrategyTwelveUniform
+from poisson_approval.profiles.ProfileCardinal import ProfileCardinal
+from poisson_approval.strategies.StrategyTwelve import StrategyTwelve
+from poisson_approval.tau_vector.TauVector import TauVector
+from poisson_approval.utils.DictPrintingInOrderIgnoringZeros import DictPrintingInOrderIgnoringZeros
+from poisson_approval.utils.SetPrintingInOrder import SetPrintingInOrder
+from poisson_approval.utils.Util import ballot_one, ballot_two, ballot_one_two, ballot_one_three, sort_ballot, \
+    product_dict, ballot_low_u, ballot_high_u, sort_weak_order, my_division
 from poisson_approval.utils.UtilCache import cached_property
 
 
@@ -24,6 +25,9 @@ class ProfileTwelve(ProfileCardinal):
         that a voter prefers candidate ``a``, then candidate ``b``, then candidate ``c``, with a utility for ``b``
         that is infinitely close to 1. In contrast, ``d_type_share['a_bc']`` is the probability that a voter prefers
         ``a`` then ``b`` then ``c``, with a utility for ``b`` that is infinitely close to 0.
+    d_weak_order_share : dict
+        E.g. ``{'a~b>c': 0.2, 'a>b~c': 0.1}``. ``d_weak_order_share['a~b>c']`` is the probability that a voter likes
+        candidates ``a`` and ``b`` equally and prefer them to candidate ``c``.
     normalization_warning : bool
         Whether a warning should be issued if the input distribution is not normalized.
     ratio_sincere : Number
@@ -33,6 +37,8 @@ class ProfileTwelve(ProfileCardinal):
         and `ratio_fanatic` must not exceed 1.
     voting_rule : str
         The voting rule. Possible values are ``APPROVAL``, ``PLURALITY`` and ``ANTI_PLURALITY``.
+    symbolic : bool
+        Whether the computations are symbolic or numeric.
 
     Notes
     -----
@@ -75,6 +81,24 @@ class ProfileTwelve(ProfileCardinal):
         {'abc', 'bac', 'cab'}
         >>> profile.is_generic_in_rankings  # Are all rankings there?
         False
+        >>> profile.analyzed_strategies_pure
+        Equilibria:
+        <abc: a, bac: b, cab: ac> ==> b (FF)
+        <abc: a, bac: ab, cab: c> ==> a (D)
+        <abc: ab, bac: b, cab: utility-dependent> ==> b (FF)
+        <BLANKLINE>
+        Non-equilibria:
+        <abc: a, bac: b, cab: c> ==> b (FF)
+        <abc: a, bac: b, cab: utility-dependent> ==> b (FF)
+        <abc: a, bac: ab, cab: ac> ==> a (D)
+        <abc: a, bac: ab, cab: utility-dependent> ==> a (D)
+        <abc: ab, bac: b, cab: c> ==> b (FF)
+        <abc: ab, bac: b, cab: ac> ==> b (FF)
+        <abc: ab, bac: ab, cab: c> ==> a, b (FF)
+        <abc: ab, bac: ab, cab: ac> ==> a (D)
+        <abc: ab, bac: ab, cab: utility-dependent> ==> a (D)
+        >>> print(profile.analyzed_strategies_pure.winners_at_equilibrium)
+        a, b
 
     In the following example, one third of the voters are sincere:
 
@@ -92,22 +116,45 @@ class ProfileTwelve(ProfileCardinal):
         >>> tau = profile.tau(strategy)
         >>> print(tau)
         <a: 1/15, ab: 1/30, ac: 1/10, b: 3/5, c: 1/5> ==> b
+
+    The profile can include weak orders:
+
+        >>> profile = ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(6, 10)},
+        ...                         d_weak_order_share={'a~b>c': Fraction(3, 10)})
+        >>> profile
+        ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(3, 5)}, d_weak_order_share={'a~b>c': Fraction(3, 10)})
+        >>> print(profile)
+        <ab_c: 1/10, b_ac: 3/5, a~b>c: 3/10> (Condorcet winner: b)
     """
 
-    def __init__(self, d_type_share, normalization_warning=True, ratio_sincere=0, ratio_fanatic=0,
-                 voting_rule=APPROVAL):
-        super().__init__(ratio_sincere=ratio_sincere, ratio_fanatic=ratio_fanatic, voting_rule=voting_rule)
-        # Populate the dictionary
+    def __init__(self, d_type_share, d_weak_order_share=None, normalization_warning=True,
+                 ratio_sincere=0, ratio_fanatic=0, voting_rule=APPROVAL, symbolic=False):
+        super().__init__(ratio_sincere=ratio_sincere, ratio_fanatic=ratio_fanatic, voting_rule=voting_rule,
+                         symbolic=symbolic)
+        if d_weak_order_share is None:
+            d_weak_order_share = dict()
+        # Populate the dictionaries of types and weak orders
         self.d_type_share = DictPrintingInOrderIgnoringZeros({t: 0 for t in TWELVE_TYPES})
-        for t, share in d_type_share.items():
-            self.d_type_share[t] += share
+        self._d_weak_order_share = DictPrintingInOrderIgnoringZeros({
+            weak_order: 0 for weak_order in WEAK_ORDERS_WITHOUT_INVERSIONS})
+        for t, share in itertools.chain(d_type_share.items(), d_weak_order_share.items()):
+            try:
+                self.d_type_share[t] += share
+            except KeyError:
+                self._d_weak_order_share[sort_weak_order(t)] += share
         # Normalize if necessary
-        total = sum(self.d_type_share.values())
-        if not isclose(total, 1.):
+        total = sum(self.d_type_share.values()) + sum(self._d_weak_order_share.values())
+        if not self.ce.look_equal(total, 1):
             if normalization_warning:
-                warnings.warn("Warning: profile is not normalized, I will normalize it.")
+                warnings.warn(NORMALIZATION_WARNING)
             for t in self.d_type_share.keys():
-                self.d_type_share[t] /= total
+                self.d_type_share[t] = my_division(self.d_type_share[t], total)
+            for weak_order in self._d_weak_order_share.keys():
+                self._d_weak_order_share[weak_order] = my_division(self._d_weak_order_share[weak_order], total)
+
+    @cached_property
+    def d_weak_order_share(self):
+        return self._d_weak_order_share
 
     def have_ranking_with_utility_above_u(self, ranking, u):
         """Share of voters who have a given ranking and strictly above a given utility for their middle candidate.
@@ -126,11 +173,11 @@ class ProfileTwelve(ProfileCardinal):
             >>> profile.have_ranking_with_utility_above_u(ranking='cab', u=1)
             0
         """
-        high_u = self.d_type_share[ranking[:2] + '_' + ranking[2:]]  # E.g. ab_c
-        low_u = self.d_type_share[ranking[:1] + '_' + ranking[1:]]   # E.g. a_bc
         if u == 1:
             return 0
+        high_u = self.d_type_share[ranking[:2] + '_' + ranking[2:]]  # E.g. ab_c
         if u == 0:
+            low_u = self.d_type_share[ranking[:1] + '_' + ranking[1:]]  # E.g. a_bc
             return high_u + low_u
         return high_u
 
@@ -166,25 +213,27 @@ class ProfileTwelve(ProfileCardinal):
             >>> profile.have_ranking_with_utility_below_u(ranking='cab', u=0)
             0
         """
-        high_u = self.d_type_share[ranking[:2] + '_' + ranking[2:]]  # E.g. ab_c
-        low_u = self.d_type_share[ranking[:1] + '_' + ranking[1:]]   # E.g. a_bc
-        if u == 1:
-            return high_u + low_u
         if u == 0:
             return 0
+        low_u = self.d_type_share[ranking[:1] + '_' + ranking[1:]]   # E.g. a_bc
+        if u == 1:
+            high_u = self.d_type_share[ranking[:2] + '_' + ranking[2:]]  # E.g. ab_c
+            return high_u + low_u
         return low_u
 
     def __repr__(self):
         """
-        >>> from fractions import Fraction
-        >>> profile = ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(6, 10),
-        ...                          'c_ab': Fraction(2, 10), 'ca_b': Fraction(1, 10)},
-        ...                         ratio_sincere=Fraction(1, 10), ratio_fanatic=Fraction(1, 5))
-        >>> profile
-        ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(3, 5), 'c_ab': Fraction(1, 5), \
+            >>> from fractions import Fraction
+            >>> profile = ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(6, 10),
+            ...                          'c_ab': Fraction(2, 10), 'ca_b': Fraction(1, 10)},
+            ...                         ratio_sincere=Fraction(1, 10), ratio_fanatic=Fraction(1, 5))
+            >>> profile
+            ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(3, 5), 'c_ab': Fraction(1, 5), \
 'ca_b': Fraction(1, 10)}, ratio_sincere=Fraction(1, 10), ratio_fanatic=Fraction(1, 5))
         """
         arguments = repr(self.d_type_share)
+        if self.contains_weak_orders:
+            arguments += ', d_weak_order_share=%r' % self.d_weak_order_share
         if self.ratio_sincere > 0:
             arguments += ', ratio_sincere=%r' % self.ratio_sincere
         if self.ratio_fanatic > 0:
@@ -195,14 +244,20 @@ class ProfileTwelve(ProfileCardinal):
 
     def __str__(self):
         """
-        >>> from fractions import Fraction
-        >>> profile = ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(6, 10),
-        ...                          'c_ab': Fraction(2, 10), 'ca_b': Fraction(1, 10)},
-        ...                         ratio_sincere=Fraction(1, 10), ratio_fanatic=Fraction(1, 5))
-        >>> print(profile)
-        <ab_c: 1/10, b_ac: 3/5, c_ab: 1/5, ca_b: 1/10> (Condorcet winner: b) (ratio_sincere: 1/10) (ratio_fanatic: 1/5)
+            >>> from fractions import Fraction
+            >>> profile = ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(6, 10),
+            ...                          'c_ab': Fraction(2, 10), 'ca_b': Fraction(1, 10)},
+            ...                         ratio_sincere=Fraction(1, 10), ratio_fanatic=Fraction(1, 5))
+            >>> print(profile)
+            <ab_c: 1/10, b_ac: 3/5, c_ab: 1/5, ca_b: 1/10> (Condorcet winner: b) (ratio_sincere: 1/10) \
+(ratio_fanatic: 1/5)
         """
-        result = '<%s>' % str(self.d_type_share)[1:-1]
+        contents = []
+        if self.contains_rankings:
+            contents.append(str(self.d_type_share)[1:-1])
+        if self.contains_weak_orders:
+            contents.append(str(self.d_weak_order_share)[1:-1])
+        result = '<' + ', '.join(contents) + '>'
         if self.is_profile_condorcet:
             result += ' (Condorcet winner: %s)' % self.condorcet_winners
         if self.ratio_sincere > 0:
@@ -240,6 +295,7 @@ class ProfileTwelve(ProfileCardinal):
         """
         return (isinstance(other, ProfileTwelve)
                 and self.d_type_share == other.d_type_share
+                and self.d_weak_order_share == other.d_weak_order_share
                 and self.ratio_sincere == other.ratio_sincere
                 and self.ratio_fanatic == other.ratio_fanatic
                 and self.voting_rule == other.voting_rule)
@@ -267,11 +323,16 @@ class ProfileTwelve(ProfileCardinal):
         best_signature = []
         for perm in XYZ_PERMUTATIONS:
             d_test = {translate(t, perm): share for t, share in self.d_type_share.items()}
+            d_test.update({sort_weak_order(translate(weak_order, perm)): share
+                           for weak_order, share in self.d_weak_order_share.items()})
             signature_test = [d_test[t] for t in XYZ_TWELVE_TYPES]
+            signature_test += [d_test[weak_order] for weak_order in XYZ_WEAK_ORDERS_WITHOUT_INVERSIONS]
             if signature_test > best_signature:
                 best_signature = signature_test
                 best_d = d_test
         return ProfileTwelve({t: best_d[xyz_t] for t, xyz_t in zip(TWELVE_TYPES, XYZ_TWELVE_TYPES)},
+                             {weak_order: best_d[xyz_weak_order] for weak_order, xyz_weak_order in zip(
+                                 WEAK_ORDERS_WITHOUT_INVERSIONS, XYZ_WEAK_ORDERS_WITHOUT_INVERSIONS)},
                              ratio_sincere=self.ratio_sincere, ratio_fanatic=self.ratio_fanatic,
                              voting_rule=self.voting_rule)
 
@@ -286,8 +347,14 @@ class ProfileTwelve(ProfileCardinal):
             ...                          'c_ab': Fraction(2, 10), 'ca_b': Fraction(1, 10)})
             >>> profile.has_majority_type
             True
+
+        This does NOT include weak orders:
+
+            >>> profile = ProfileTwelve({'ab_c': Fraction(1, 10)}, d_weak_order_share={'b>a~c': Fraction(9, 10)})
+            >>> profile.has_majority_type
+            False
         """
-        return max(self.d_type_share.values()) > 0.5
+        return max(self.d_type_share.values()) > Fraction(1, 2)
 
     # Has full support
     @cached_property
@@ -344,7 +411,7 @@ class ProfileTwelve(ProfileCardinal):
             <ab: 1/10, ac: 1/10, b: 3/5, c: 1/5> ==> b
         """
         assert self.voting_rule == strategy.voting_rule
-        t = {ballot: 0 for ballot in BALLOTS_WITHOUT_INVERSIONS}
+        t = self.d_ballot_share_weak_voters_sincere.copy()  # For weak orders, strategic = sincere
         for ranking, ballot in strategy.d_ranking_ballot.items():
             if self.d_ranking_share[ranking] == 0:
                 continue
@@ -354,7 +421,7 @@ class ProfileTwelve(ProfileCardinal):
                 t[ballot_high_u(ranking, self.voting_rule)] += self.have_ranking_with_utility_above_u(ranking, u=.5)
             else:
                 t[ballot] += self.d_ranking_share[ranking]
-        return TauVector(t, voting_rule=self.voting_rule)
+        return TauVector(t, voting_rule=self.voting_rule, symbolic=self.symbolic)
 
     def is_equilibrium(self, strategy):
         """Whether a strategy is an equilibrium.
@@ -407,39 +474,15 @@ class ProfileTwelve(ProfileCardinal):
                     return EquilibriumStatus.NOT_EQUILIBRIUM
         return status
 
-    @cached_property
-    def analyzed_strategies(self):
-        """AnalyzedStrategies : Analyzed strategies of the profile.
+    @property
+    def strategies_pure(self):
+        """Iterator: pure strategies of the profile.
 
-        Examples
-        --------
-            >>> from fractions import Fraction
-            >>> profile = ProfileTwelve({'ab_c': Fraction(1, 10), 'b_ac': Fraction(6, 10),
-            ...                          'c_ab': Fraction(2, 10), 'ca_b': Fraction(1, 10)})
-            >>> profile.analyzed_strategies
-            Equilibria:
-            <abc: a, bac: b, cab: ac> ==> b (FF)
-            <abc: a, bac: ab, cab: c> ==> a (D)
-            <abc: ab, bac: b, cab: utility-dependent> ==> b (FF)
-            <BLANKLINE>
-            Non-equilibria:
-            <abc: a, bac: b, cab: c> ==> b (FF)
-            <abc: a, bac: b, cab: utility-dependent> ==> b (FF)
-            <abc: a, bac: ab, cab: ac> ==> a (D)
-            <abc: a, bac: ab, cab: utility-dependent> ==> a (D)
-            <abc: ab, bac: b, cab: c> ==> b (FF)
-            <abc: ab, bac: b, cab: ac> ==> b (FF)
-            <abc: ab, bac: ab, cab: c> ==> a, b (FF)
-            <abc: ab, bac: ab, cab: ac> ==> a (D)
-            <abc: ab, bac: ab, cab: utility-dependent> ==> a (D)
-            >>> print(profile.winners_at_equilibrium)
-            a, b
+        Yields
+        ------
+        StrategyTwelve
+            All possible pure strategies of the profile.
         """
-        equilibria = []
-        utility_dependent = []
-        inconclusive = []
-        non_equilibria = []
-
         def possible_strategies(share_ranking_1, share_ranking_12, strategy_1, strategy_12):
             if share_ranking_1 > 0 and share_ranking_12 > 0:
                 return [strategy_1, strategy_12, UTILITY_DEPENDENT]
@@ -457,19 +500,43 @@ class ProfileTwelve(ProfileCardinal):
         }
 
         for d_ranking_strategy in product_dict(d_ranking_possible_strategies):
-            strategy = StrategyTwelve(d_ranking_strategy, profile=self)
-            status = strategy.is_equilibrium
-            if status == EquilibriumStatus.EQUILIBRIUM:
-                equilibria.append(strategy)
-            elif status == EquilibriumStatus.UTILITY_DEPENDENT:  # pragma: no cover
-                utility_dependent.append(strategy)
-                warnings.warn('Met a utility-dependent case: \nprofile = %r\nstrategy = %r' % (self, strategy))
-            elif status == EquilibriumStatus.INCONCLUSIVE:  # pragma: no cover
-                inconclusive.append(strategy)
-                warnings.warn('Met an inconclusive case: \nprofile = %r\nstrategy = %r' % (self, strategy))
-            else:
-                non_equilibria.append(strategy)
-        return AnalyzedStrategies(equilibria, utility_dependent, inconclusive, non_equilibria)
+            yield StrategyTwelve(d_ranking_strategy, profile=self)
+
+    @property
+    def strategies_group(self):
+        raise NotImplementedError
+
+    @classmethod
+    def order_and_label(cls, t):
+        r"""Order and label of a discrete type.
+
+        Cf. :meth:`Profile.order_and_label`.
+
+        Examples
+        --------
+            >>> ProfileTwelve.order_and_label('ab_c')
+            ('abc', '$r(ab\\_c)$')
+            >>> ProfileTwelve.order_and_label('a~b>c')
+            ('a~b>c', '$r(a\\sim b>c)$')
+        """
+        if len(t) == 4:
+            return t.replace('_', ''), ('$r(%s)$' % t).replace('_', '\\_')
+        else:
+            return cls.order_and_label_weak(t)
+
+    @classmethod
+    def random_strategy(cls, **kwargs):
+        """Random strategy.
+
+        This is a default generator of random strategies. It is used, for example, in
+        :class:`ProfileCardinal.iterated_voting`.
+
+        Returns
+        -------
+        StrategyThreshold
+            Uses :class:`GeneratorStrategyTwelveUniform`.
+        """
+        return GeneratorStrategyTwelveUniform(**kwargs)()
 
 
 def make_property_type_share(t, doc):
