@@ -1,7 +1,7 @@
 from pytest import fixture
 from fractions import Fraction
-from poisson_approval import ProfileTwelve, StrategyTwelve, StrategyThreshold, EquilibriumStatus, PLURALITY, \
-    ANTI_PLURALITY, UTILITY_DEPENDENT, TauVector
+from poisson_approval import ProfileTwelve, StrategyTwelve, StrategyOrdinal, EquilibriumStatus, \
+    PLURALITY, ANTI_PLURALITY, TauVector, initialize_random_seeds
 
 
 def test_iterative_voting_verbose():
@@ -147,5 +147,28 @@ TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'bc': Fraction(2, 5)}))
         (None, TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'b': Fraction(2, 5)}))
         >>> profile._initializer('fanatic')
         (None, TauVector({'a': Fraction(2, 5), 'b': Fraction(2, 5), 'c': Fraction(1, 5)}))
+        >>> initialize_random_seeds()
+        >>> profile._initializer('random')
+        (StrategyTwelve({'abc': 'ab', 'acb': 'ac', 'bac': 'b', 'bca': 'bc', 'cab': 'utility-dependent', 'cba': 'bc'}), \
+TauVector({'ab': Fraction(2, 5), 'ac': Fraction(1, 5), 'bc': Fraction(2, 5)}))
+    """
+    pass
+
+
+def test_strategy_weak_order():
+    """
+        >>> profile = ProfileTwelve({'ab_c': Fraction(1, 7), 'a~b>c': Fraction(2, 7), 'c>a~b': Fraction(4, 7)})
+        >>> strategy = StrategyOrdinal({'abc': 'a'}, profile=profile)
+        >>> print(strategy.tau)
+        <a: 1/7, ab: 2/7, c: 4/7> ==> c
+    """
+    pass
+
+
+def test_random_strategy():
+    """
+        >>> initialize_random_seeds()
+        >>> print(ProfileTwelve.random_strategy())
+        <abc: ab, acb: ac, bac: b, bca: bc, cab: utility-dependent, cba: bc>
     """
     pass
